import 'package:analyzer/dart/element/element.dart';
import 'package:like_sealed_gen/src/util.dart';

class ContentFormat {
  String build(String import, String className, List<ClassElement> types) {
    var content = "";
    content += "import '$import';\n";
    content += "import 'package:like_sealed/sealed_switch.dart';\n";
    content += _class(className, types);
    content += _switch(className, types);

    return content;
  }

  String _class(String className, List<ClassElement> types) {
    return """
   class ${className}s {
   ${_methods(types)}
   }
   """;
  }

  String _switch(String className, List<ClassElement> types) {
    String typeChecker = "";
    String caseFunction = "";
    for (var type in types) {
      final typeName = type.name;

      typeChecker += "type is $typeName? on$typeName(type): ";
      caseFunction += "T on$typeName($typeName ${typeName.firstToLower});";
    }

    typeChecker += "null";

    return """
    abstract class ${className}Switch<T> implements SealedSwitch<State,T> {
      T switchCase($className type)=>$typeChecker;
      $caseFunction
    }
    """;
  }

  String _methods(List<ClassElement> types) {
    final methods = List<String>();
    for (var type in types) {
      if (type.isAbstract) {
        continue;
      }
      for (var constructor in type.constructors) {
        if (constructor.isAbstract) {
          continue;
        }
        var functionName = type.name.firstToLower;
        var className = type.name;

        if (constructor.name.isNotEmpty) {
          functionName = functionName + constructor.name.firstToUpper;
          className = className + "." + constructor.name;
        }

        final textConstructor = constructor.parameters.map(
          (parameter) {
            if (parameter.isNamed) {
              return parameter.name + " : " + parameter.name;
            } else {
              return parameter.name;
            }
          },
        ).join(" , ");

        final function = """
        static $className $functionName(${constructor.parameters.join(" , ")}){
          return $className($textConstructor);
        } 
        """;

        methods.add(function);
      }
    }
    return methods.join("\n");
  }
}
